C  File:  n1_r_bmp.for

C  Autor:  G. Schnell

C  16.03.2008

C  Mandelbrot/Julia - Mengen im BMP-File-Format

      IMPLICIT NONE

      CHARACTER*132 Name        ! Input aus:   n1_r_bmp.dat
      CHARACTER*80  Name_LS     ! Output z.B.: n1_r_bmp_Name.ls
      CHARACTER*80  Name_BMP    ! Output  "  : n1_r_bmp_Name.bmp
      COMMON / OUTPUT_DATEIEN / Name, Name_ls, Name_bmp


      ! -------  Common BITMAP Anfang  --------------------

C  Max. Anzahl Pixel horizontal und vertical

      INTEGER*4 iBit_Max
!     PARAMETER ( iBit_Max =    64 )   ! Minimum    1078 = 14 + 40 + 256*4
!     PARAMETER ( iBit_Max =  2048 )   ! 4   MB, standard
!     PARAMETER ( iBit_Max =  4096 )   ! 16  MB 
!     PARAMETER ( iBit_Max =  8192 )   ! 64  MB
      PARAMETER ( iBit_Max = 16384 )   ! Maxi 268 436 534 = 1078 + 16384^2 
    

      CHARACTER*1 cbmfh(14)    ! ----- BitMapFileHeader -----
      CHARACTER*2 cbfType      ! BitMapFileType immer 'BM'=16973
                               ! 'B'=66; 'M'=77; 66*256+77=16973
      CHARACTER*4 cbfSize      ! BitMapFileSize in Bytes
      CHARACTER*2 cbfReserved1 ! = 0
      CHARACTER*2 cbfReserved2 ! = 0
      CHARACTER*4 cbfOffBits   ! offset from the beginning of the
                               ! file to the bitmap data
      EQUIVALENCE( cbmfh( 1), cbfType      )
      EQUIVALENCE( cbmfh( 3), cbfSize      )
      EQUIVALENCE( cbmfh( 7), cbfReserved1 )
      EQUIVALENCE( cbmfh( 9), cbfReserved2 )
      EQUIVALENCE( cbmfh(11), cbfOffBits   )

      CHARACTER*1 cbmih(40)      ! ----- BitMapInfoHeader -----
      CHARACTER*4 cbiSize        ! size of the InfoHeader in Bytes (40)
      CHARACTER*4 cbiWidth       ! width of the image in pixels
      CHARACTER*4 cbiHeight      ! height of the image in pixels
      CHARACTER*2 cbiPlanes      ! number of planes of target device(1)
      CHARACTER*2 cbiBitCount    ! number of bits per pixel (1,4,8,24)
      CHARACTER*4 cbiCompression ! type of compression, 0=no compr.
      CHARACTER*4 cbiSizeImage   ! size of image data in Bytes. =0 if
                                 ! no compression
      CHARACTER*4 cbiXPelsPerM   ! horizontal pixels per meter on the
                                 ! designated targer device, usually = 0
      CHARACTER*4 cbiYPelsPerM   ! vertical pixels per meter on the
                                 ! designated targer device, usually = 0
      CHARACTER*4 cbiClrUsed     ! number of colors used in the bitmap
                                 ! if set to zero the number of colors
                                 ! is calculated using the biBitCount
      CHARACTER*4 cbiClrImportant ! number of colors that are important
                                 ! for the bitmap, if set to zero, all
                                 ! colors are important
      EQUIVALENCE( cbmih( 1), cbiSize         )
      EQUIVALENCE( cbmih( 5), cbiWidth        )
      EQUIVALENCE( cbmih( 9), cbiHeight       )
      EQUIVALENCE( cbmih(13), cbiPlanes       )
      EQUIVALENCE( cbmih(15), cbiBitCount     )
      EQUIVALENCE( cbmih(17), cbiCompression  )
      EQUIVALENCE( cbmih(21), cbiSizeImage    )
      EQUIVALENCE( cbmih(25), cbiXPelsPerM    )
      EQUIVALENCE( cbmih(29), cbiYPelsPerM    )
      EQUIVALENCE( cbmih(33), cbiClrUsed      )
      EQUIVALENCE( cbmih(37), cbiClrImportant )

      CHARACTER*4 cFcolors(256)   ! Farbtabelle fuer biBitCount = 8

      CHARACTER*1 cFbits (iBit_Max) ! 1 Bit == 1 Byte fuer biBitCount=8

      COMMON / BITMAP / cbmfh, cbmih, cFcolors, cFbits

      ! -------  Common BITMAP Ende   ---------------------


      ! -------  Common CHAOS Anfang  ---------------------

      INTEGER*4 iPixels    ! Anzahl gewuenschte Pixel in X/Y-Richtung
      INTEGER*4 iX, iY     ! Anzahl Pixel in X/Y-Richtung
      INTEGER*4 iGitter    ! Distanz X-/Y-Gitter, 1 bis 100 oder Null
      REAL*8 dX            ! Delta X/Y ( gleich, sonst Verzerrung )
      REAL*8 Xra, Xre      ! reeller Zahlenbereich aufsteigend
      REAL*8 Xia, Xie      ! imag.   Zahlenbereich aufsteigend

      LOGICAL*4 JULIA      ! .T. = Julia-Menge, .F. = Mandelbrot
      REAL*8 CJ(2)         ! Konstante fuer Julia-Menge
      INTEGER*4 iT_MAX     ! Max. Anzahl Iterationen pro Punkt
      INTEGER*4 iPow       ! Potenz in: Z(n+1) = Z(n)^iPow + c
      REAL*8    EPS        ! Abbruchkriterium |z|^2 = 4  z.B.
      REAL*8    Y          ! Vorzeichen-Parameter Nplex
      REAL*8    E0,E1      ! Reduktions-Parameter Nplex

      INTEGER*4 iAnzC      ! Anzahl Farb-Bereiche ( 2 bis 256 )
      INTEGER*4 iFindex(2,256) ! Anfangs-/End-Indices Farbbereiche
      INTEGER*4 iFcolor(3,256) ! 256 mal (rot, gruen, blau)-Farben

      COMMON / CHAOS / iPixels, iX, iY, iGitter, dX, Xra, Xre, Xia, Xie,
     +                 iT_MAX, iPow, EPS, Y, E0, E1, JULIA, CJ,
     +                 iAnzC, iFindex, iFcolor

      ! -------  Common CHAOS Ende   ----------------------

      INTEGER*4 iBit_MaxV     ! max. Anzahl Pixel vertikal

      INTEGER*4 IO_LI         ! I/O unit Error-Liste
      INTEGER*4 IO_DAT        ! I/O unit Data

      CHARACTER*80 FILE_DAT   ! Datafile

      INTEGER*4 I, iERR
      REAL*8 lx, li

      LOGICAL*4 Lnegativ      ! RGB als CMY

      iBit_MaxV = iBit_Max    ! Max. Anzahl Pixel vertikal

      IO_DAT = 19   ! I/O Units setzen
      IO_LI  =  6   !

      ! Dateinamen:
      FILE_DAT = 'n1_r_bmp.dat'    ! Input - Data

      OPEN( UNIT=IO_DAT, FILE = FILE_DAT )

C  Daten von Datafile lesen

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Output-Namen .LS, .BMP
      READ(IO_DAT, '(A)' ) Name
      CALL UP_NAMEN( Name, Name_ls, Name_bmp, iERR )
      IF( iERR .NE. 0 ) THEN
         WRITE(IO_LI,*) 'Fehler im Dateinamen fuer: .LS, .BMP'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! .T.= Julia-Menge, .F.=Mandelb.
      READ(IO_DAT, * ) JULIA

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Konstante zu Julia-Menge
      READ(IO_DAT, * ) CJ(1), CJ(2)

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Max. Potenz in z^iPow + c
      READ(IO_DAT, * ) iPow
      IF( ( iPow .LT. 2 ) .OR. ( iPow .GT. 128 ) ) THEN
         WRITE(IO_LI,*) 'iPow = ', iPow, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Abbruchkriterium EPS
      READ(IO_DAT, * ) EPS              ! Standard = 4
      IF( ( EPS .LT. 0.1 ) .OR. ( EPS .GT. 100.0 ) ) THEN
         WRITE(IO_LI,*) 'EPS = ', EPS, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      ! Nplex-Parameter y und e0, e1 eingeben.'
      CALL UP_IN_POS( IO_DAT, IO_LI )
      READ(IO_DAT, * ) Y, E0, E1
      IF( DABS(Y) .LT. 0.001D0 ) THEN
         WRITE(IO_LI,*) 'n1-Parameter Y kleiner .001 .'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      ! Reellen Bereich Xra bis Xre einlesen ( aufsteigend ).
      CALL UP_IN_POS( IO_DAT, IO_LI )
      READ(IO_DAT, * ) Xra, Xre
      IF( Xra .GE. Xre ) THEN
         WRITE(IO_LI,*) 'Reeller Zahlenbereich nicht aufsteigend.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      ! "Imagin." Bereich Xia bis Xie einlesen ( aufsteigend ).
      CALL UP_IN_POS( IO_DAT, IO_LI )
      READ(IO_DAT, * ) Xia, Xie
      IF( Xia .GE. Xie ) THEN
         WRITE(IO_LI,*) 'Imagin. Zahlenbereich nicht aufsteigend.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Distanz Gitterlinien
      READ(IO_DAT, * ) iGitter          ! in X/Y - Richtung
      IF( ( iGitter .LT. 0 ) .OR. ( iGitter .GT. 100 ) ) THEN
         WRITE(IO_LI,*) 'Gitter-Distanz = ', iGitter, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! gewuenschte Anzahl Pixels
      READ(IO_DAT, * ) iPixels          ! in X/Y - Richtung
      IF( ( iPixels .LT. 4 ) .OR. ( iPixels .GT. iBit_Max ) ) THEN
         WRITE(IO_LI,*) 'Anzahl Pixel = ', iPixels, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
      I = MOD( iPixels, 4 )
      IF( I .NE. 0 ) THEN
         WRITE(IO_LI,*) 'Warning: Pixel = ', iPixels, ' nicht 4 * k'
         iPixels = iPixels+3 - MOD( iPixels+3, 4 )
         PAUSE 'Bitte Return-Taste druecken'
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Max. Anzahl Iterationen
      READ(IO_DAT, * ) iT_MAX
      IF( ( iT_MAX .LT. 2 ) .OR. ( iT_MAX .GT. 256 ) ) THEN
         WRITE(IO_LI,*) 'iT_MAX = ', iT_MAX, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      ! RGB in CMY (negativ) umrechnen ?
      CALL UP_IN_POS( IO_DAT, IO_LI )
      READ(IO_DAT, * ) Lnegativ

      ! Anzahl Farb-Bereiche, mindestens 2, hoechstens 256
      CALL UP_IN_POS( IO_DAT, IO_LI )
      READ(IO_DAT, * ) iAnzC
      IF( ( iAnzC .LT. 2 ) .OR. ( iAnzC .GT. 256 ) ) THEN
         WRITE(IO_LI,*) 'Anzahl Farbbereiche:'
         WRITE(IO_LI,*) 'iAnzC = ', iAnzC, ' nicht erlaubt.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      ! Farb-Bereiche und Farben rot, gruen, blau
      CALL UP_IN_POS( IO_DAT, IO_LI )
      DO I = 1, iAnzC
         READ(IO_DAT, * ) iFindex(1,I), iFindex(2,I),
     +                    iFcolor(1,I), iFcolor(2,I), iFcolor(3,I)
      END DO

      ! Testen ob Bereiche ohne Luecken bzw. Ueberschneidungen
      IF( iFindex(1,1) .NE. 1 ) THEN
         WRITE(IO_LI,*) 'Anzahl Farbbereiche:'
         WRITE(IO_LI,*) 'iFindex(1,1) nicht gleich 1.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      IF( iFindex(2,iAnzC) .NE. 256 ) THEN
         WRITE(IO_LI,*) 'Anzahl Farbbereiche:'
         WRITE(IO_LI,*) 'iFindex(2,iAnzC) nicht gleich 256.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF

      DO I = 1, iAnzC
      IF( iFindex(2,I) .LT. iFindex(1,I) ) THEN
         WRITE(IO_LI,*) 'Anzahl Farbbereiche:'
         WRITE(IO_LI,*) 'Bereichsende kleiner Bereichsanfang.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
      END DO

      DO I = 2, iAnzC
      IF( iFindex(2,I-1)+1 .NE. iFindex(1,I) ) THEN
         WRITE(IO_LI,*) 'Anzahl Farbbereiche:'
         WRITE(IO_LI,*) 'Bereichsende + 1 ungleich Bereichsanfang.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
      END DO

      ! Testen ob Farben zwischen 0 und 255
      DO I = 1, iAnzC
      IF( (iFcolor(1,I).LT.0) .OR. (iFcolor(1,I).GT. 255) ) THEN
         WRITE(IO_LI,*) 'Farbe rot nicht zwischen 0 und 255.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
      IF( (iFcolor(2,I).LT.0) .OR. (iFcolor(2,I).GT. 255) ) THEN
         WRITE(IO_LI,*) 'Farbe gruen nicht zwischen 0 und 255.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
      IF( (iFcolor(3,I).LT.0) .OR. (iFcolor(3,I).GT. 255) ) THEN
         WRITE(IO_LI,*) 'Farbe blau nicht zwischen 0 und 255.'
         PAUSE 'Bitte Return-Taste druecken'
         STOP
      END IF
         IF( Lnegativ ) THEN
            iFcolor(1,I) = 255 - iFcolor(1,I)
            iFcolor(2,I) = 255 - iFcolor(2,I)
            iFcolor(3,I) = 255 - iFcolor(3,I)
         END IF
      END DO

      ! Berechnen dX und eventuelle Begrenzung iPixels
      lx = Xre - Xra
      li = Xie - Xia
      IF( lx .GT. li ) THEN
         dX = lx / ( iPixels - 1.0D0 )
         iX = iPixels
         iY = ( li + dx/2.0D0 ) / dx
         iY = iY + 1
      ELSE
         IF( iPixels .GT. iBit_MaxV ) THEN
             iPixels = iBit_MaxV
             WRITE(IO_LI,*) 'warning, iPixels begrenzt auf iBit_MaxV.'
             PAUSE 'Bitte Return-Taste druecken'
         END IF
         dX = li / ( iPixels - 1.0D0 )
         iY = iPixels
         iX = ( lx + dx/2.0D0 ) / dx
         iX = iX + 1
         iX = iX+3 - MOD( iX+3, 4 )   ! iX = 4 * k
      END IF

      CLOSE( IO_DAT )

      ! Erstellen Apfelmaennchen und Varianten mit Param. Y und E0, E1
      CALL Mandelbrot( Lnegativ )

      STOP

      END
C  ---------------------------------------------------
C  FILE_DAT = 'n1_r_bmp.dat' fuers Lesen positionieren
C  und Kommentar- und Leer-Zeilen ueberlesen.

      SUBROUTINE UP_IN_POS( IO_IN, IO_LI )

      IMPLICIT NONE

      INTEGER*4 IO_IN, IO_LI, I
      CHARACTER*136 REC

   10 READ( IO_IN, '(A)', ERR=800, END=900 ) REC
      IF( REC(1:1) .EQ. '!' ) GO TO 10   ! Kommentarzeile ueberlesen
      DO 20, I = 1, 136
         IF( REC(I:I) .NE. ' ' ) THEN
            BACKSPACE IO_IN              ! Datenzeile gefunden
            RETURN                       !      "        "
         END IF
   20 CONTINUE
      GO TO 10                           ! Leerzeile ueberlesen

  800 WRITE( IO_LI, * ) 'Error beim Lesen von n1_r_bmp.dat'
      STOP

  900 WRITE( IO_LI, * ) 'EOF erreicht beim Lesen von n1_r_bmp.dat'
      STOP
      END
C  ---------------------------------------------------------------
C  Farbe zum Index in CH1 ( 0 bis 255 ) durch Gitterfarbe ersetzen
C  iAnzC   = Anzahl Farbbereiche
C  iFindex = Feld mit Farbbereichen
C  iFcolor = Feld mit Farben der Farbbereiche

      SUBROUTINE UP_Gitter( iAnzC, iFindex, iFcolor, CH1 )

      IMPLICIT NONE

      SAVE

      INTEGER*4 iAnzC, iFindex(2,256), iFcolor(3,256)
      CHARACTER*1 CH1

      INTEGER*4 I, iP, iF, iMin, iSum
      INTEGER*4 irot, igruen, iblau

      iP = iCHAR( CH1 ) + 1   ! Punkt in einem Farbbereich
      DO I = 1, iAnzC
         IF( (iP.GE.iFindex(1,I)).AND.(iP.LE.iFindex(2,I))) THEN
            irot   = iFcolor(1,I)
            igruen = iFcolor(2,I)
            iblau  = iFcolor(3,I)
            iF = I
            GO TO 100   ! Farbe zu CH1 gefunden
         END IF
      END DO
  100 CONTINUE
      iMin = 0
      DO I = 1, iAnzC
         iSum = IABS( iFcolor(1,I) - irot   )
     +        + IABS( iFcolor(2,I) - igruen )
     +        + IABS( iFcolor(3,I) - iblau  )
         IF( iSum .GT. iMin ) THEN
             iMin = iSum
             iF   = I
         END IF
      END DO
      iP = iFindex(1,iF) - 1
      CH1 = CHAR( iP )

      RETURN

      END
C  --------------------------
C  Variationen zu Mandelbrot.

      SUBROUTINE Mandelbrot( Lnegativ )

      IMPLICIT NONE

      LOGICAL*4 Lnegativ

      ! -------  Common BITMAP Anfang  --------------------

C  Max. Anzahl Pixel horizontal und vertical

      INTEGER*4 iBit_Max
!     PARAMETER ( iBit_Max =    64 )   ! Minimum    1078 = 14 + 40 + 256*4
!     PARAMETER ( iBit_Max =  2048 )   ! 4   MB, standard
!     PARAMETER ( iBit_Max =  4096 )   ! 16  MB 
!     PARAMETER ( iBit_Max =  8192 )   ! 64  MB
      PARAMETER ( iBit_Max = 16384 )   ! Maxi 268 436 534 = 1078 + 16384^2 
    

      CHARACTER*1 cbmfh(14)    ! ----- BitMapFileHeader -----
      CHARACTER*2 cbfType      ! BitMapFileType immer 'BM'=16973
                               ! 'B'=66; 'M'=77; 66*256+77=16973
      CHARACTER*4 cbfSize      ! BitMapFileSize in Bytes
      CHARACTER*2 cbfReserved1 ! = 0
      CHARACTER*2 cbfReserved2 ! = 0
      CHARACTER*4 cbfOffBits   ! offset from the beginning of the
                               ! file to the bitmap data
      EQUIVALENCE( cbmfh( 1), cbfType      )
      EQUIVALENCE( cbmfh( 3), cbfSize      )
      EQUIVALENCE( cbmfh( 7), cbfReserved1 )
      EQUIVALENCE( cbmfh( 9), cbfReserved2 )
      EQUIVALENCE( cbmfh(11), cbfOffBits   )

      CHARACTER*1 cbmih(40)      ! ----- BitMapInfoHeader -----
      CHARACTER*4 cbiSize        ! size of the InfoHeader in Bytes (40)
      CHARACTER*4 cbiWidth       ! width of the image in pixels
      CHARACTER*4 cbiHeight      ! height of the image in pixels
      CHARACTER*2 cbiPlanes      ! number of planes of target device(1)
      CHARACTER*2 cbiBitCount    ! number of bits per pixel (1,4,8,24)
      CHARACTER*4 cbiCompression ! type of compression, 0=no compr.
      CHARACTER*4 cbiSizeImage   ! size of image data in Bytes. =0 if
                                 ! no compression
      CHARACTER*4 cbiXPelsPerM   ! horizontal pixels per meter on the
                                 ! designated targer device, usually = 0
      CHARACTER*4 cbiYPelsPerM   ! vertical pixels per meter on the
                                 ! designated targer device, usually = 0
      CHARACTER*4 cbiClrUsed     ! number of colors used in the bitmap
                                 ! if set to zero the number of colors
                                 ! is calculated using the biBitCount
      CHARACTER*4 cbiClrImportant ! number of colors that are important
                                 ! for the bitmap, if set to zero, all
                                 ! colors are important
      EQUIVALENCE( cbmih( 1), cbiSize         )
      EQUIVALENCE( cbmih( 5), cbiWidth        )
      EQUIVALENCE( cbmih( 9), cbiHeight       )
      EQUIVALENCE( cbmih(13), cbiPlanes       )
      EQUIVALENCE( cbmih(15), cbiBitCount     )
      EQUIVALENCE( cbmih(17), cbiCompression  )
      EQUIVALENCE( cbmih(21), cbiSizeImage    )
      EQUIVALENCE( cbmih(25), cbiXPelsPerM    )
      EQUIVALENCE( cbmih(29), cbiYPelsPerM    )
      EQUIVALENCE( cbmih(33), cbiClrUsed      )
      EQUIVALENCE( cbmih(37), cbiClrImportant )

      CHARACTER*4 cFcolors(256)   ! Farbtabelle fuer biBitCount = 8

      CHARACTER*1 cFbits (iBit_Max) ! 1 Bit == 1 Byte fuer biBitCount=8

      COMMON / BITMAP / cbmfh, cbmih, cFcolors, cFbits

      ! -------  Common BITMAP Ende   ---------------------


      ! -------  Common CHAOS Anfang  ---------------------

      INTEGER*4 iPixels    ! Anzahl gewuenschte Pixel in X/Y-Richtung
      INTEGER*4 iX, iY     ! Anzahl Pixel in X/Y-Richtung
      INTEGER*4 iGitter    ! Distanz X-/Y-Gitter, 1 bis 100 oder Null
      REAL*8 dX            ! Delta X/Y ( gleich, sonst Verzerrung )
      REAL*8 Xra, Xre      ! reeller Zahlenbereich aufsteigend
      REAL*8 Xia, Xie      ! imag.   Zahlenbereich aufsteigend

      LOGICAL*4 JULIA      ! .T. = Julia-Menge, .F. = Mandelbrot
      REAL*8 CJ(2)         ! Konstante fuer Julia-Menge
      INTEGER*4 iT_MAX     ! Max. Anzahl Iterationen pro Punkt
      INTEGER*4 iPow       ! Potenz in: Z(n+1) = Z(n)^iPow + c
      REAL*8    EPS        ! Abbruchkriterium |z|^2 = 4  z.B.
      REAL*8    Y          ! Vorzeichen-Parameter Nplex
      REAL*8    E0,E1      ! Reduktions-Parameter Nplex

      INTEGER*4 iAnzC      ! Anzahl Farb-Bereiche ( 2 bis 256 )
      INTEGER*4 iFindex(2,256) ! Anfangs-/End-Indices Farbbereiche
      INTEGER*4 iFcolor(3,256) ! 256 mal (rot, gruen, blau)-Farben

      COMMON / CHAOS / iPixels, iX, iY, iGitter, dX, Xra, Xre, Xia, Xie,
     +                 iT_MAX, iPow, EPS, Y, E0, E1, JULIA, CJ,
     +                 iAnzC, iFindex, iFcolor

      ! -------  Common CHAOS Ende   ----------------------

      INTEGER*4 I, J, Ja, Je, K, Ie
      INTEGER*4 iGL/ 15 /    ! max. Anzahl Gitter-Koordinaten
      REAL*8 rX, rY
      REAL*8 C0, C1, DET
      REAL*8 Fa(2), A0, A1
      EQUIVALENCE ( Fa(1), A0 )
      EQUIVALENCE ( Fa(2), A1 )
      REAL*8 AP(2), P0, P1
      EQUIVALENCE ( AP(1), P0 )
      EQUIVALENCE ( AP(2), P1 )

      INTEGER*2   I2           !
      CHARACTER*2 CH2          !
      EQUIVALENCE( I2, CH2 )   !

      INTEGER*4   I4           !
      CHARACTER*4 CH4          !
      EQUIVALENCE( I4, CH4 )   !

      INTEGER*4 IO_DAT/ 20 /   ! Output- Unit BitMap
      INTEGER*4 IO_LS/ 22 /    ! Output- Unit InfoListe

      CHARACTER*132 Name        ! Input aus n1_r_bmp.dat
      CHARACTER*80  Name_LS     ! Output n1_r_bmp_name.ls
      CHARACTER*80  Name_BMP    ! Output n1_r_bmp_name.bmp
      COMMON / OUTPUT_DATEIEN / Name, Name_ls, Name_bmp

      INTEGER*4 iFls(256)   ! Verteilung Anzahl Iterationen

      REAL*8 Sum/ 0.0D0 /   ! Gesamte Anzahl Iterationen

      ! Reset iFls
      DO I = 1, 256
         iFls(I) = 0
      END DO

      ! Setzen BitMapFileHeader -------------

      cbfType = 'BM'

      ! BitMapFileSize in Bytes
      I4 = 14 + 40 + 256*4 + iX*iY
      cbfSize(1:1) = CH4(1:1)
      cbfSize(2:2) = CH4(2:2)
      cbfSize(3:3) = CH4(3:3)
      cbfSize(4:4) = CH4(4:4)

      I2 = 0
      cbfReserved1(1:1) = CH2(1:1)
      cbfReserved1(2:2) = CH2(2:2)

      I2 = 0
      cbfReserved2(1:1) = CH2(1:1)
      cbfReserved2(2:2) = CH2(2:2)

      I4 = 1078   ! 14 + 40 + 256*4 = 1078
      cbfOffBits(1:1) = CH4(1:1)
      cbfOffBits(2:2) = CH4(2:2)
      cbfOffBits(3:3) = CH4(3:3)
      cbfOffBits(4:4) = CH4(4:4)

      ! Setzen BitMapInfoHeader -------------

      I4 = 40
      cbiSize(1:1) = CH4(1:1)
      cbiSize(2:2) = CH4(2:2)
      cbiSize(3:3) = CH4(3:3)
      cbiSize(4:4) = CH4(4:4)

      I4 = iX
      cbiWidth(1:1) = CH4(1:1)
      cbiWidth(2:2) = CH4(2:2)
      cbiWidth(3:3) = CH4(3:3)
      cbiWidth(4:4) = CH4(4:4)

      I4 = iY
      cbiHeight(1:1) = CH4(1:1)
      cbiHeight(2:2) = CH4(2:2)
      cbiHeight(3:3) = CH4(3:3)
      cbiHeight(4:4) = CH4(4:4)

      I2 = 1
      cbiPlanes(1:1) = CH2(1:1)
      cbiPlanes(2:2) = CH2(2:2)

      I2 = 8
      cbiBitCount(1:1) = CH2(1:1)
      cbiBitCount(2:2) = CH2(2:2)

      I4 = 0
      cbiCompression  = CH4   ! geht nur fuer I4 = 0 !
      cbiSizeImage    = CH4   !   "
      cbiXPelsPerM    = CH4   !   "
      cbiYPelsPerM    = CH4   !   "
      cbiClrUsed      = CH4   !   "
      cbiClrImportant = CH4   !   "

      ! Setzen Farbtabelle --------------

      DO I = 1, iAnzC
         Ja = iFindex(1,I)
         Je = iFindex(2,I)
         DO J = Ja, Je
            cFcolors(J)(3:3) = CHAR( iFcolor(1,I) )   ! rot (blau)
            cFcolors(J)(2:2) = CHAR( iFcolor(2,I) )   ! gruen
            cFcolors(J)(1:1) = CHAR( iFcolor(3,I) )   ! blau (rot)
            cFcolors(J)(4:4) = CHAR( 0 )              ! Null
         END DO
      END DO


      OPEN( UNIT = IO_DAT, FILE = Name_bmp,           ! Bitmap-File
     +      FORM = 'UNFORMATTED', RECORDTYPE = 'FIXED' )

      WRITE( IO_DAT ) cbmfh       ! BitMapFileHeader
      WRITE( IO_DAT ) cbmih       ! BitMapInfoHeader
      WRITE( IO_DAT ) cFcolors    ! Farftabelle

      cFbits(iX)   = CHAR( 0 )    ! Falls iX nicht 4*k ( muss nicht )
      cFbits(iX-1) = CHAR( 0 )
      cFbits(iX-2) = CHAR( 0 )

      C1 = Xia - dX              ! Imag-Teil Start-Initialisierung

      DO I = 1, iY               ! Anzahl Zeilen

         C1 = C1 + dX            ! Imag-Teil

         C0 = Xra - dX           ! Real-Teil Start-Initialisierung

         DO J = 1, iX            ! Anzahl Spalten

            C0 = C0 + dX

            A0 = C0
            A1 = C1

            DO K = 1, iT_MAX

               IF( iPow .EQ. 2 ) THEN
                  P0 = A0*A0 - E0*A1*A1
                  P1 = Y*(2.0D0*A0*A1-E1*A1*A1)
               ELSE
                  CALL A_iPow( Fa, AP )   ! a^iPow
               END IF
               IF( JULIA ) THEN   ! Julia-Menge
                  A0 = P0 + CJ(1)
                  A1 = P1 + CJ(2)
               ELSE
                  A0 = P0 + C0    ! Mandelbrot-Menge
                  A1 = P1 + C1
               END IF
               ! Laenge von a ( ohne Wurzelziehen )
               DET = A0*A0 + A1*A1
               IF( DET .GT. EPS ) THEN
                   cFbits(J) = CHAR( K-1 )   ! Abbruch
                   iFls(K) = iFls(k) + 1
                   GO TO 100
               END IF
            END DO   ! K-Schleife
            cFbits(J) = CHAR( IT_Max-1 )     ! kein Abbruch
            iFls(IT_MAX) = iFls(IT_MAX) + 1
  100       CONTINUE

         END DO   ! J-Schleife

         ! Gitterliniern zeichnen
         IF( iGitter .GT. 0 ) THEN
            IF( MOD(I,iGitter) .EQ. 0 ) THEN   ! Zeile als Gitterlinie
               DO J = 1, iX
                  ! Farbwechsel in cFbits
                  CALL UP_Gitter(iAnzC, iFindex, iFcolor, cFbits(J) )
               END DO
            ELSE
               DO J = 1, iX
                IF( MOD(J,iGitter) .EQ. 0 ) THEN   ! Spalte als Gitterl.
                  ! Farbwechsel in cFbits
                  CALL UP_Gitter(iAnzC, iFindex, iFcolor, cFbits(J) )
                END IF
               END DO
            END IF
         END IF

         WRITE(IO_DAT) (cFbits(J), J=1,iX )

      END DO      ! I-Schleife

      CLOSE( IO_DAT )

      ! Informationen zur BitMap-Datei ausgeben  -----------------
      OPEN( UNIT=IO_LS, FILE=Name_ls )

      WRITE(IO_LS, * ) Name_ls
      WRITE(IO_LS, * )

      IF( JULIA ) THEN
         WRITE(IO_LS, * ) 'Julia-Menge:'
         WRITE(IO_LS,1000) CJ(1), CJ(2)
         WRITE(IO_LS, * )
      ELSE
         WRITE(IO_LS, * ) 'Mandelbrot-Menge:'
         WRITE(IO_LS, * )
      END IF

      WRITE(IO_LS,*) 'n1-Parameter:'
      WRITE(IO_LS,*) 'y  = ', y
      WRITE(IO_LS,*) 'e0 = ', e0
      WRITE(IO_LS,*) 'e1 = ', e1
      WRITE(IO_LS,*)
      WRITE(IO_LS,2010) iPow
      WRITE(IO_LS,*)
      WRITE(IO_LS,2020) iT_Max
      WRITE(IO_LS,*)
      WRITE(IO_LS,*) 'Abbruchgroesse EPS = ', EPS
      WRITE(IO_LS,*)
      WRITE(IO_LS,*) 'Distanz Gitterlinien = ', iGitter
      WRITE(IO_LS,*)
      WRITE(IO_LS,*) 'Gewuenschte Anzahl Pixel = ',iPixels
      WRITE(IO_LS,*) 'Anzahl Pixel horizontal  = ',iX
      WRITE(IO_LS,*) 'Anzahl Pixel vertikal    = ',iY
      WRITE(IO_LS,*)
      WRITE(IO_LS,1010) Xra, Xre
      WRITE(IO_LS,1020) Xia, Xie
      WRITE(IO_LS,*)
      WRITE(IO_LS,1050) dX
      IF( iGitter .GT. 1 ) THEN   ! feste Anzahl Gitterkoordinaten
         WRITE(IO_LS,*)           ! Anzahl = iGL
         WRITE(IO_LS,*) 'Gitter-Koordinaten:'
         WRITE(IO_LS,1060)
         IF( iX .GT. iY ) THEN
            Ie = iX
         ELSE
            Ie = iY
         END IF
         Ie = Ie - MOD( Ie, iGitter )
         IF( Ie .GT. iGL*iGitter ) Ie = iGL*iGitter
         IF( Ie .EQ. 0 ) Ie = iGitter   ! mindestens eine Koordinate
         I = 0
         WRITE(IO_LS,1070) I, Xra, Xia
         DO I = iGitter, Ie, iGitter
            rX = Xra + dX * (I-1.0D0)
            rY = Xia + dX * (I-1.0D0)
            WRITE(IO_LS,1070) I, rX, rY
         END DO
      END IF

      IF( .NOT. Lnegativ ) THEN     ! RGB
         WRITE(IO_LS,*)
         WRITE(IO_LS,1030) iAnzC
         DO I = 1, iAnzC
         WRITE(IO_LS,1040) iFindex(1,I), iFindex(2,I),
     +                     iFcolor(1,I), iFcolor(2,I),iFcolor(3,I)
         END DO
         WRITE(IO_LS,*)
      ELSE                           ! RGB in CMY ( negativ )
         WRITE(IO_LS,*)
         WRITE(IO_LS,1030) iAnzC
         WRITE(IO_LS,*) 'RGB-Werte in CMY gewandelt (negativ)'
         DO I = 1, iAnzC
         WRITE(IO_LS,1040) iFindex(1,I), iFindex(2,I),
     +         255-iFcolor(1,I), 255-iFcolor(2,I),255-iFcolor(3,I),
     +             iFcolor(1,I),     iFcolor(2,I),    iFcolor(3,I)
         END DO
         WRITE(IO_LS,*)
      END IF

      ! Tabelle Anzahl Iterationen
      WRITE(IO_LS,*) 'Iterations-Tabelle:'
      Ie = IT_MAX+7 - MOD(IT_MAX+7,8)
      DO I = 1, Ie, 8
         WRITE(IO_LS,2000) I, I+7, (iFls(J),J=I,I+7)
      END DO

      DO I = 1, IT_MAX
         SUM = SUM + I * DBLE( iFls(I) )
      END DO
      WRITE(IO_LS,*)
      WRITE(IO_LS,1100) SUM
      WRITE(IO_LS,1110) SUM/(IX*IY)

      CLOSE( IO_LS )

      RETURN

 1000 FORMAT('Konstante = (',E14.7,' , ',E14.7,' )')
 1010 FORMAT('Bereich reell    :',2E14.7)
 1020 FORMAT('Bereich imaginaer:',2E14.7)
 1030 FORMAT('Anzahl Farbbereiche:',I4)
 1040 FORMAT(I3,' - ',I3,':   ',3I4,3X,3I4)
 1050 FORMAT('dX = dY = ',D22.15)
 1060 FORMAT('Gitter                      Xr                      Xi'
     +)
 1070 FORMAT(I6,2X,1P,D22.15,2X,1P,D22.15)
 1100 FORMAT('Gesamte  Anzahl Iterationen:',1P,D18.10)
 1110 FORMAT('Mittlere Anzahl Iterationen:',1P,D14.6)

 2000 FORMAT(I3,'-',I3,':',8I8)
 2010 FORMAT('iPower = ',I3)
 2020 FORMAT('iT_Max = ',I3)

      END
C  ------------------------------
C  Matrix Ma zu Nplex a erstellen
C  Die Laenge des Nplex a ist 2
C  Y ist Vorzeichen-Parameter
C  E0, E1 sind Reduktions-Parameter
C  A0, A1 stehen in Fa

      SUBROUTINE GEN_MA( Fa, Ma, Y, E0, E1 )

      IMPLICIT NONE

      REAL*8 Fa(2), Ma(2,2), Y, E0, E1

      MA(1,1) =  Fa(1)
      MA(1,2) = -Fa(2) * E0
      MA(2,1) =  Y *  Fa(2)
      MA(2,2) =  Y * ( Fa(1) - Fa(2) * E1 )

      RETURN

      END
C  ------------------------------------
C  NPLEX ^ iPow
C  Koeffizienten des Nplex a in Fa( 2 )

C  AP = ( p0 , p1 ) = ( a0 , a1 )^iPow

      SUBROUTINE A_iPow( Fa, AP )

      IMPLICIT NONE

      ! -------  Common CHAOS Anfang  ---------------------

      INTEGER*4 iPixels    ! Anzahl gewuenschte Pixel in X/Y-Richtung
      INTEGER*4 iX, iY     ! Anzahl Pixel in X/Y-Richtung
      INTEGER*4 iGitter    ! Distanz X-/Y-Gitter, 1 bis 100 oder Null
      REAL*8 dX            ! Delta X/Y ( gleich, sonst Verzerrung )
      REAL*8 Xra, Xre      ! reeller Zahlenbereich aufsteigend
      REAL*8 Xia, Xie      ! imag.   Zahlenbereich aufsteigend

      LOGICAL*4 JULIA      ! .T. = Julia-Menge, .F. = Mandelbrot
      REAL*8 CJ(2)         ! Konstante fuer Julia-Menge
      INTEGER*4 iT_MAX     ! Max. Anzahl Iterationen pro Punkt
      INTEGER*4 iPow       ! Potenz in: Z(n+1) = Z(n)^iPow + c
      REAL*8    EPS        ! Abbruchkriterium |z|^2 = 4  z.B.
      REAL*8    Y          ! Vorzeichen-Parameter Nplex
      REAL*8    E0,E1      ! Reduktions-Parameter Nplex

      INTEGER*4 iAnzC      ! Anzahl Farb-Bereiche ( 2 bis 256 )
      INTEGER*4 iFindex(2,256) ! Anfangs-/End-Indices Farbbereiche
      INTEGER*4 iFcolor(3,256) ! 256 mal (rot, gruen, blau)-Farben

      COMMON / CHAOS / iPixels, iX, iY, iGitter, dX, Xra, Xre, Xia, Xie,
     +                 iT_MAX, iPow, EPS, Y, E0, E1, JULIA, CJ,
     +                 iAnzC, iFindex, iFcolor

      ! -------  Common CHAOS Ende   ----------------------

      REAL*8 Fa( 2 )       ! Nplex a
      REAL*8 AP( 2 )       ! a^iPow, Ergebnis

      INTEGER*4 G, RM

      REAL*8 MA( 2, 2 )    ! Nplex a als Matrix mit e und y
      REAL*8 QT( 2, 2 )
      REAL*8 ST( 2, 2 )
      REAL*8 ZT( 2, 2 )    ! Zwischenspeicher

      LOGICAL*4 START

      ! Nplex a als Matrix Ma ( mit e und y )
      CALL GEN_MA( Fa, Ma, Y, E0, E1 )

      IF( iPow .EQ. 2 ) THEN       ! iPow = 2, Fertig, Normalfall
         ! AP = Ma * Fa
         CALL MLT_M_V( Ma, Fa, AP )
         RETURN
      ELSE IF( iPow .EQ. 3 ) THEN  ! iPow = 3
         CALL DMLTM( Ma, Ma, ZT )
         CALL MLT_M_V( ZT, Fa, AP )
         RETURN
      ELSE IF( iPow .EQ. 4 ) THEN  ! iPow = 4
         CALL DMLTM( Ma, Ma, QT )
         CALL DMLTM( QT, Ma, ZT )
         CALL MLT_M_V( ZT, Fa, AP )
         RETURN
      ELSE IF( iPow .EQ. 5 ) THEN  ! iPow = 5
         CALL DMLTM( Ma, Ma, QT )
         CALL DMLTM( QT, QT, ZT )
         CALL MLT_M_V( ZT, Fa, AP )
         RETURN
      END IF

      ! Nun allgemein fuer iPow > 5

      START = .TRUE.
      G = iPow - 1    ! G = Exponent, G wird veraendert

C     QT = Ma * Ma     ! QT = ( a0, a1 )^2
      CALL DMLTM( MA, MA, QT )

      RM = MOD( G, 2 )

      IF( RM .EQ. 0 ) THEN
         ! ST = E
         ST(1,1) = 1.0D0
         ST(1,2) = 0.0D0
         ST(2,1) = 0.0D0
         ST(2,2) = 1.0D0
      ELSE
         ! ST = MA
         CALL DMOVEM( ST, MA )
      END IF

  100 CONTINUE                       ! Schleife G Anfang
      G = ( G - RM ) / 2

      IF( G .EQ. 0 ) GO TO 200   ! fertig

      RM = MOD( G, 2 )

      IF( START ) THEN
         START = .FALSE.
      ELSE
         !  Q  = Q * Q
         CALL DMLTM( QT, QT, ZT )
         CALL DMOVEM( QT, ZT )
      END IF

      IF( RM .NE. 0 ) THEN
         !  S  = Q * S
         CALL DMLTM( QT, ST, ZT )
         CALL DMOVEM( ST, ZT )
      END IF

      GO TO 100                      ! Schleife G Ende

  200 CONTINUE

      ! AP = ST * Fa
      CALL MLT_M_V( ST, Fa, AP )

      RETURN

      END
C  -----------------------
C  Zuweisen Matrix B auf A
C  A(2,2) = B(2,2)

      SUBROUTINE DMOVEM( A, B )

      IMPLICIT NONE

      REAL*8 A(2,2), B(2,2)

      A(1,1) = B(1,1)
      A(1,2) = B(1,2)
      A(2,1) = B(2,1)
      A(2,2) = B(2,2)
      RETURN
      END
C  -------------------------------------
C  C(2,2) = A(2,2) * B(2,2)     (REAL*8)

      SUBROUTINE DMLTM( A, B, C )

      IMPLICIT NONE

      REAL*8 A(2,2), B(2,2), C(2,2)

      C(1,1) = A(1,1)*B(1,1) + A(1,2)*B(2,1)
      C(1,2) = A(1,1)*B(1,2) + A(1,2)*B(2,2)
      C(2,1) = A(2,1)*B(1,1) + A(2,2)*B(2,1)
      C(2,2) = A(2,1)*B(1,2) + A(2,2)*B(2,2)
      RETURN
      END
C  -------------------------------------------------
C  Multiplizieren Matrix Ma mit Vektor b, Ma * b = c

      SUBROUTINE MLT_M_V( Ma, b, c )

      IMPLICIT NONE

      REAL*8 Ma(2,2), b(2), c(2)

      c(1) = Ma(1,1)*b(1) + Ma(1,2)*b(2)
      c(2) = Ma(2,1)*b(1) + Ma(2,2)*b(2)

      RETURN
      END
C  -------------------------------------------------------
      SUBROUTINE UP_NAMEN( Name, Name_ls, Name_bmp, iERR )

      IMPLICIT NONE

      INTEGER*4 iERR   ! iERR = 0 = okay
      CHARACTER*132 Name
      CHARACTER*80  Name_ls
      CHARACTER*80  Name_bmp

      INTEGER*4 iLang/ 72 /   ! max. Laenge ohne Extension
      INTEGER*4 LCH/ 132 /
      INTEGER*4 iANF/1/, iEND/132/, iPOS, I

      LOGICAL*4 PR_ALPH, PR_NUM

      ! Kommentar nach Filenamen abschneiden
      CALL FIND( Name, iANF, iEND, '!', iPOS )
      IF( iPOS .GT. 0 ) LCH = iPOS - 1

      CALL DEL_SPACE( Name, LCH )   ! Spaces nach rechts
      iEND = LCH
      CALL NOTFINDB( Name, iANF, iEND, ' ', iPOS )  ! iPOS=Laenge Name
      LCH = iPOS
      IF( LCH .EQ. 0 ) THEN
         iERR = 1    ! Name ist Space
         RETURN
      ELSE IF( LCH .GT. iLang ) THEN
         iERR = 2    ! Name laenger als erlaubt
         RETURN
      END IF
!     CALL UPPER_CASE( Name, LCH )  ! optional
      IF( .NOT. PR_ALPH( Name(1:1) ) ) THEN
         iERR = 3    ! 1. Zeichen kein Buchstabe
         RETURN
      END IF
      DO I = 2, LCH
         IF( ( Name(I:I) .EQ. '_' ) .OR.
     +       ( Name(I:I) .EQ. '-' ) .OR. 
     +         PR_ALPH(Name(I:I))   .OR.
     +         PR_NUM (Name(I:I))   ) THEN
            ! okay
         ELSE
            iERR = 4  ! unzulaessiges Zeichen
            RETURN
         END IF
      END DO

      iERR = 0
      Name_ls = Name(1:LCH )   ! Rest = Space
      Name_ls(LCH+1:LCH+3) = '.ls'
      Name_bmp = Name(1:LCH )   ! Rest = Space
      Name_bmp(LCH+1:LCH+4) = '.bmp'

      RETURN
      END
C  --------------------------------
C  Spaces in String FCH eliminieren
C  Spaces wandern ans Ende von FCH

      SUBROUTINE DEL_SPACE( FCH, LCH )

      IMPLICIT NONE

      INTEGER*4 LCH, I, IP
      CHARACTER*(*) FCH

      IP = 0
      DO 10, I = 1, LCH
      IF( FCH(I:I) .EQ. ' ' ) THEN
      ! NOTHING
      ELSE
      IP = IP + 1
      FCH(IP:IP) = FCH(I:I)
      END IF
   10 CONTINUE
      IF( (IP.EQ.0) .OR. (IP.EQ.LCH) ) RETURN
      DO 20, I = IP+1, LCH
   20 FCH(I:I) = ' '
      RETURN
      END
C  ----------------------------------------------------
C  Umsetzen Klein- in Gross-Buchstaben in Feld FCH(LCH)

      SUBROUTINE UPPER_CASE( FCH, LCH )

      IMPLICIT NONE

      INTEGER*4 LCH, I
      CHARACTER*(*) FCH

      DO 10, I = 1, LCH
      IF( ( ICHAR(FCH(I:I)) .GE. '141'O )  .AND.
     +    ( ICHAR(FCH(I:I)) .LE. '172'O ) ) THEN
      FCH(I:I) = CHAR( ICHAR( FCH(I:I) ) - '40'O )
      END IF
   10 CONTINUE
      RETURN
      END
C  -------------------------------
C  Pruefen ob Zeichen Alphabetisch

      LOGICAL*4 FUNCTION PR_ALPH( CH1 )

      IMPLICIT NONE

      CHARACTER*1 CH1

      IF( ( (CH1.GE.'A').AND.(CH1.LE.'Z') ) .OR.
     +    ( (CH1.GE.'a').AND.(CH1.LE.'z') ) ) THEN
      PR_ALPH = .TRUE.
      ELSE
      PR_ALPH = .FALSE.
      END IF

      RETURN
      END
C  --------------------------
C  Tabs durch Spaces ersetzen

      SUBROUTINE TAB_SP( FCH, LCH )

      IMPLICIT NONE

      INTEGER*4 LCH, I
      CHARACTER*(*) FCH

      DO 10, I = 1, LCH
      IF( FCH(I:I) .EQ. '<11>' ) FCH(I:I) = ' '
   10 CONTINUE

      RETURN
      END
C  =================================================
C  Laenge ( 1 bis N ) eines Zeichenstrings bestimmen

      INTEGER*4 FUNCTION LENGTH_1( REC, N )

      IMPLICIT NONE

      INTEGER*4 N, I
      CHARACTER*(*) REC

      LENGTH_1 = 1   ! Falls REC = SPACE

      DO 10, I = N, 1, -1
         IF( REC(I:I) .NE. ' ' ) THEN
            LENGTH_1 = I
            RETURN
         END IF
   10 CONTINUE

      END
C  -----------------------------------------------
C  Pruefen ob Zeichen CHAR in String STR enthalten
C  STR  = String
C  IANF = Position in STR ab der gesucht wird
C  IEND = Position in STR bis zu der gesucht wird
C  CHAR = Zeichen nach dem gesucht wird
C  IPOS = 1. Position des Zeichens in STR oder 0

      SUBROUTINE FIND( STR, IANF, IEND, CHAR, IPOS )

      IMPLICIT NONE

      INTEGER*4 IANF, IEND, IPOS
      INTEGER*4 I0
      PARAMETER ( I0=0 )
      CHARACTER*1 STR(*), CHAR

      DO 10 IPOS = IANF, IEND
      IF( STR(IPOS) .EQ. CHAR ) GO TO 900
   10 CONTINUE
      IPOS = I0
  900 RETURN
      END
C  -----------------------------------------------------------
C  Rueckwaerts pruefen ob Zeichen CHAR in String STR enthalten
C  STR  = String
C  IANF = Position in STR bis zu der rueckw. gesucht wird
C  IEND = Position in STR ab der rueckw. gesucht wird
C  CHAR = Zeichen nach dem gesucht wird
C  IPOS = Letzte Position des Zeichens in STR oder 0

      SUBROUTINE FINDB( STR, IANF, IEND, CHAR, IPOS )

      IMPLICIT NONE

      INTEGER*4 IANF, IEND, IPOS
      INTEGER*4 I0, I1M
      PARAMETER ( I0=0, I1M = -1 )
      CHARACTER*1 STR(*), CHAR

      DO 10, IPOS = IEND, IANF, I1M
      IF( STR(IPOS) .EQ. CHAR ) GO TO 900
   10 CONTINUE
      IPOS = I0
  900 RETURN
      END
C  ---------------------------------------------
C  Suchen 1. Zeichen ungleich CHAR in String STR
C  STR  = String
C  IANF = Position in STR ab der gesucht wird
C  IEND = Position in STR bis zu der grsucht wird
C  CHAR = Zeichen nach dem nicht gesucht wird
C  IPOS = 1. Position des Zeichens ungleich CHAR in STR oder 0

      SUBROUTINE NOTFIND( STR, IANF, IEND, CHAR, IPOS )

      IMPLICIT NONE

      INTEGER*4 IANF, IEND, IPOS
      INTEGER*4 I0
      PARAMETER ( I0=0 )
      CHARACTER*1 STR(*), CHAR
 
      DO 10 IPOS = IANF, IEND
      IF( STR(IPOS) .NE. CHAR ) GO TO 900
   10 CONTINUE
      IPOS = I0
  900 RETURN
      END
C  ---------------------------------------------------------
C  Rueckwaerts suchen 1. Zeichen ungleich CHAR in String STR
C  Hiermit wird die Laenge eines Zeichenstrings (ohne '<00>')!
C  bestimmt
C  STR  = String
C  IANF = Position in STR bis zu der gesucht wird
C  IEND = Position in STR ab der gesucht wird
C  CHAR = Zeichen nach dem nicht gesucht wird
C  IPOS = 1. Position des Zeichens ungleich CHAR in STR oder 0

      SUBROUTINE NOTFINDB( STR, IANF, IEND, CHAR, IPOS )

      IMPLICIT NONE

      INTEGER*4 IANF, IEND, IPOS
      INTEGER*4 I0, I1M
      PARAMETER ( I0=0, I1M = -1 )
      CHARACTER*1 STR(*), CHAR

      DO 10, IPOS = IEND, IANF, I1M
      IF( STR(IPOS) .NE. CHAR ) GO TO 900
   10 CONTINUE
      IPOS = I0
  900 RETURN
      END
C  -----------------------------
C  Zuweisen String2 nach String1
C  STR1   = String1
C  IANF1  = erste Position in STR1
C  IEND1  = letzte Position in STR1
C  STR2   = String2
C  IANF2  = erste Position in STR2

      SUBROUTINE MOVE( STR1, IANF1, IEND1, STR2, IANF2 )

      IMPLICIT NONE

      INTEGER*4 IANF1, IEND1, IANF2, I, J
      INTEGER*4 I1
      PARAMETER ( I1=1 )
      CHARACTER*1 STR1(*), STR2(*)

      J = IANF2
      DO 10 I = IANF1, IEND1
      STR1(I) = STR2(J)
      J = J + I1
   10 CONTINUE
      RETURN
      END
C  ============================================
C  Pruefen ob Zeichen numerisch ( '0' bis '9' )

      LOGICAL*4 FUNCTION PR_NUM( CH1 )

      IMPLICIT NONE

      CHARACTER*1 CH1

      IF( (CH1.GE.'0') .AND. (CH1.LE.'9') ) THEN
      PR_NUM = .TRUE.
      ELSE
      PR_NUM = .FALSE.
      END IF

      RETURN
      END
