! File: n1_s_exp_ln.for 

! 14.01.2008

! Autor:  G. Schnell

! Periodische Funktion exp(0,x+2*k*Pi) mit s-Plex umkehren
! 
! Parameterset:
! n=1, o=s, g=1, y=1, e0=1, e1=0

!  Umkehrung der periodischen Funktion:
!  y = exp( i*(x+2*k*Pi) )  0 =< x <  2 * Pi    und
!  y = exp( i*(x-2*k*Pi) )  0 => x > -2 * Pi
!  also: |x| < 2*Pi 

!  Das Argument wird in a = ( a_1, a0, a1 ) aufbereitet und
!  dann b = exp( a ) berechnet ( a0 = 0 ).
!  Aus b wird C = ln( b ) bestimmt und C mit c = F * C korrigiert
!  Die Umkehrung scheitert an den Polstellen  +-( Pi + {0,1,2,...}*2*Pi )
!  und den Nullstellen  +-( 2*Pi + {0,1,2,...}*2*Pi ) 
!  nicht aber an den gleichen Nullstellen  +-( 0 + {1,2,3,...}*2*Pi ) !

!  a ist s-Plex der Dimension d = 3.
!  Aus numerischen Gruenden wird K begrenzt, zur Zeit auf K <= 10 000

      IMPLICIT NONE

      REAL*8 a(-1:1), b(-1:1), c(-1:1)

      REAL*8 a_1, a0, a1
      EQUIVALENCE( a(-1), a_1 )
      EQUIVALENCE( a( 0), a0  )
      EQUIVALENCE( a( 1), a1  )

      REAL*8 c_1, c0, c1
      EQUIVALENCE( c(-1), c_1 )
      EQUIVALENCE( c( 0), c0  )
      EQUIVALENCE( c( 1), c1  )

      INTEGER*4 x_anf, x_end, x_delta, x
      INTEGER*4 k_anf, k_end, k_delta

      INTEGER*4 k              ! k fuer 2*k*pi

      CHARACTER*72 TXT

      REAL*8 F
      REAL*8 Pi/ 3.14159265358979 /
      REAL*8 Rad

      INTEGER*4 IO_LI         ! I/O unit Liste
      INTEGER*4 IO_DAT        ! I/O unit Data
      CHARACTER*80 FILE_LI    ! Listfile
      CHARACTER*80 FILE_DAT   ! Datafile


      IO_DAT = 19   ! I/O Units setzen
      IO_LI  = 22   !

      ! Dateinamen:
      FILE_DAT = 'n1_s_exp_ln.dat'
      FILE_LI  = 'n1_s_exp_ln.ls'

      OPEN( UNIT=IO_DAT, FILE = FILE_DAT )
      OPEN( UNIT=IO_LI,  FILE = FILE_LI )

!  Daten von Datafile lesen

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! IO_DAT auf Datenzeile pos.
      READ(IO_DAT, '(A)' ) TXT          ! Text zur Aufgabe

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! k-Wert
      READ(IO_DAT,*) k_anf, k_end, k_delta
      IF( (k_anf .LT. 0) .OR. (k_anf .GT. 10 000 ) ) THEN
          WRITE(IO_LI,*) 'k_anfang = ',k_anf,' nicht erlaubt'
          STOP
      END IF
      IF( (k_end .GT. 10 000) .OR. (k_end .LT. 0) ) THEN
          WRITE(IO_LI,*) 'k_ende = ',k_end,' nicht erlaubt'
          STOP
      END IF
      IF( k_delta .EQ. 0    ) THEN
          WRITE(IO_LI,*) 'k_delta = ',k_delta,' nicht erlaubt'
          STOP
      END IF

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Werte x in Grad
      READ(IO_DAT,*) x_anf, x_end, x_delta
      IF( x_anf .LT. -360 ) THEN        ! x bis -359 Grad korrekt !
          WRITE(IO_LI,*) 'x_anfang = ',x_anf,' nicht erlaubt'
          STOP
      END IF
      IF( x_end .GT.  360 ) THEN        ! x bis 359 Grad korrekt !
          WRITE(IO_LI,*) 'x_ende = ',x_end,' nicht erlaubt'
          STOP
      END IF
      IF( x_delta .EQ. 0    ) THEN
          WRITE(IO_LI,*) 'x_delta = ',x_delta,' nicht erlaubt'
          STOP
      END IF

      CLOSE ( IO_DAT )


      WRITE(IO_LI,*) 'Programm: n1_s_exp_ln.exe'
      WRITE(IO_LI,*)
      WRITE(IO_LI,*)
     +     'Umkehrung period. Funktion y=exp( 0, i*(x+2*k*Pi) )'
      WRITE(IO_LI,*)
      WRITE(IO_LI,*) TXT
      WRITE(IO_LI,*)
 1001 FORMAT(' k_anf, k_end, k_delta:',3I6)
      WRITE(IO_LI,1001) k_anf, k_end, k_delta
      WRITE(IO_LI,*)
 1000 FORMAT(' x_anf, x_end, x_delta:',3I6)
      WRITE(IO_LI,1000) x_anf, x_end, x_delta
      WRITE(IO_LI,*)

      Rad = Pi / 180.0D0

      WRITE(IO_LI,*) 
     +     ' a = x + 2*k*pi*sign(x), b=exp(a), C=ln(b), c=F*C=a'
      WRITE(IO_LI,*)

      DO k = k_anf, k_end, k_delta

      DO x = x_anf, x_end, x_delta

         CALL SET_a( x, k, a )       ! s_Plex in a aufbereiten

         IF( x .LT. 0 ) THEN
            IF( x .EQ. -360 ) THEN
               WRITE(IO_LI,1019) K, x, x-k*360
            ELSE IF( x .EQ. -180 ) THEN
               WRITE(IO_LI,1029) K, x, x-k*360
            ELSE
               WRITE(IO_LI,1009) K, x, x-k*360
            END IF
         ELSE
            IF( x .EQ. 360 ) THEN
               WRITE(IO_LI,1020) K, x, x+k*360
            ELSE IF( x .EQ. 180 ) THEN
               WRITE(IO_LI,1030) K, x, x+k*360
            ELSE
               WRITE(IO_LI,1010) K, x, x+k*360
            END IF
         END IF
         WRITE(IO_LI,*)

 1009 FORMAT(' K = ',I5,'   x = ',I4,'    x - k*360 = ',I8)
 1010 FORMAT(' K = ',I5,'   x = ',I4,'    x + k*360 = ',I8)
 1019 FORMAT(' K = ',I5,'   x = ',I4,'    x - k*360 = ',I8,' .f.')
 1020 FORMAT(' K = ',I5,'   x = ',I4,'    x + k*360 = ',I8,' .f.')
 1029 FORMAT(' K = ',I5,'   x = ',I4,'    x - k*360 = ',I8,' .p.')
 1030 FORMAT(' K = ',I5,'   x = ',I4,'    x + k*360 = ',I8,' .p.')



         IF( DABS(a1) .GE. DABS(a_1) ) THEN
            WRITE(IO_LI,1800) a1, a1/Rad
         ELSE
            WRITE(IO_LI,1900) a_1, -a_1/Rad
         END IF
         WRITE(IO_LI,0815) 'a', a

         CALL CALC_b( a, b )   ! Berechnen exp(a)

         WRITE(IO_LI,0815) 'b', b

         CALL CALC_C( b, C )   ! Berechnen C = ln( b )

         WRITE(IO_LI,0815) 'C', C
         CALL F_b( F, b )      ! Korrektur-Faktor zu C
         c_1 = c_1 * F         ! c := C * F
         c0  = c0  * F
         c1  = c1  * F
         WRITE(IO_LI,0815) 'c', c
         WRITE(IO_LI,*)

      END DO   ! x

      END DO   ! k


 0815 FORMAT(1X,A1,1X,1P,3D14.6)
 1800 FORMAT('       a1  = ',1P,E14.6,'  ~ ',0P,E14.6)
 1900 FORMAT('      a_1  = ',1P,E14.6,'  ~ ',0P,E14.6)

      CLOSE ( IO_LI )

      STOP

      END
!  ------------------------------------------------------
!  FILE_DAT = 'n1_s_exp_ln.dat' fuers Lesen positionieren
!  und Kommentar- und Leer-Zeilen ueberlesen.

      SUBROUTINE UP_IN_POS( IO_IN, IO_LI )

      IMPLICIT NONE

      INTEGER*4 IO_IN, IO_LI, I
      CHARACTER*136 REC

   10 READ( IO_IN, '(A)', ERR=800, END=900 ) REC
      IF( REC(1:1) .EQ. '!' ) GO TO 10   ! Kommentarzeile ueberlesen
      DO 20, I = 1, 136
         IF( REC(I:I) .NE. ' ' ) THEN
            BACKSPACE IO_IN
            RETURN                       ! Datenzeile gefunden
         END IF
   20 CONTINUE
      GO TO 10                           ! Leerzeile ueberlesen

  800 WRITE( IO_LI, * ) 'Error beim Lesen von n1_s_exp_ln.dat'
      STOP

  900 WRITE( IO_LI, * ) 'EOF erreicht beim Lesen von n1_s_exp_ln.dat'
      STOP
      END
!  -----------------------------------
!  Aufbereiten Argument a = x + 2*k*Pi

      SUBROUTINE SET_a( x, k, a )

      IMPLICIT NONE

      INTEGER*4 x, k,    ia(-1:1)

      REAL*8 a(-1:1)

      REAL*8 RAD/ 0.0 174 532 925 199 432D0 /   ! Pi/180

      IF( x .LT. 0 ) THEN
         ia(-1) = -x             ! a_1 positiv setzen !
         ia(0)  = 0
         ia(1)  = 0
      ELSE
         ia(-1) = 0
         ia(0)  = 0
         ia(1)  = x
      END IF

      a(-1) = ia(-1) + K*360     ! a_1
      a(0)  = ia(0)              ! a0
      a(1)  = ia(1)  + K*360     ! a1

      a(-1) = a(-1) * Rad        ! Grad in Radian
      a(0)  = a(0)  * Rad        !
      a(1)  = a(1)  * Rad        !

      RETURN

      END
!  ----------------------------------
!  Berechnen b = exp( a ), ( a0 = 0 )

      SUBROUTINE CALC_b( a, b )

      IMPLICIT NONE

      REAL*8 a(-1:1), b(-1:1)

      REAL*8 da, ds

      da = a(1) - a(-1)         ! a1 - a_1

      IF( da .EQ. 0.0D0 ) THEN
         b(1)  = a(1)
         b(0)  = DCOS( da )
         b(-1) = a(-1)
      ELSE
         ds    = DSIN( da )
         b(1)  = a(1)*ds/da
         b(0)  = DCOS( da )
         b(-1) = b(1) - ds
      END IF
!      de = DEXP( a0 )         ! fuer: a0 # 0
!      b(-1) = b(-1) * de      !
!      b(0)  = b(0)  * de      !
!      b(1)  = b(1)  * de      !

      RETURN

      END
!  -----------------------------------------
      ! Berechnen C = ln( b ) = ln( exp(a) )

      SUBROUTINE CALC_C( b, C )

      IMPLICIT NONE

      REAL*8 b(-1:1), C(-1:1)

      REAL*8 db, ds
 
      db = b(1) - b(-1)        ! b1 - b_1

      IF( db .EQ. 0.0D0 ) THEN

         C(1)  = b(1)
         C(0)  = 0.0D0
         C(-1) = b(-1)

      ELSE

         ds = DASIN( db )

         C(1)  = b(1)*ds/db
         C(0)  = 0.0D0          ! Vereinfachung
         C(-1) = c(1) - ds

      END IF

      RETURN

      END
!  ---------------------------------------------------
!  Korrektur-Wert F(b) zu C, ( c := C * F ),  F Skalar

      SUBROUTINE F_b( F, b )

      IMPLICIT NONE

      REAL*8 F, b(-1:1)

      REAL*8 EPS/ 1.0D-13 /
      REAL*8 Pi / 3. 141 592 653 589 79 /
      REAL*8 sin, cos, x

      LOGICAL*4 Lsin, Lcos

      cos = b(0)
      sin = b(1) - b(-1)           ! Argument fuer arcsin

      ! Korrektur von sin aus numerischen Gruenden (DASIN)
      IF( sin .LT. -1.0D0 ) sin = -1.0D0
      IF( sin .GT.  1.0D0 ) sin =  1.0D0

      IF( cos .GE. 0.0D0 ) THEN
         Lcos = .TRUE.
      ELSE
         Lcos = .FALSE.
      END IF

      IF( sin .GE. 0.0D0 ) THEN
         Lsin = .TRUE.
      ELSE
         Lsin = .FALSE.
      END IF

      ! x negativ, wenn | b_1 | > | b1 |

      IF( DABS( b(-1) ) .GT. DABS( b(1) )  ) THEN   ! x neg.

         IF( Lcos .AND. (.NOT. Lsin) ) THEN
            F = 1.0D0
            RETURN
         END IF

         IF( DABS(cos) .GE. 1.0D0-EPS ) THEN   ! Polstelle ?
            IF( Lsin ) THEN
               F = -1.0D0 / ( EPS * EPS )
            ELSE
               F =  1.0D0 / ( EPS * EPS )
            END IF        
            RETURN                             
         END IF

         IF( .NOT. Lcos ) THEN
            x = -Pi - DASIN( sin )
            F = x / ( -Pi - x )
         ELSE 
            x = -2.0D0*Pi + DASIN( sin )
            F = -x / ( -2.0D0*Pi - x )
         END IF

      ELSE                         ! x positiv !

         IF( Lcos .AND. Lsin ) THEN
            F = 1.0D0
            RETURN
         END IF

         IF( DABS(cos) .GE. 1.0D0-EPS ) THEN   ! Polstelle ?
            IF( Lsin ) THEN
               F =  1.0D0 / ( EPS * EPS )
            ELSE
               F = -1.0D0 / ( EPS * EPS )
            END IF
            RETURN                            
         END IF

         IF( .NOT. Lcos ) THEN
            x = Pi - DASIN( sin )
            F = x / ( Pi - x )
         ELSE 
            x = 2.0D0*Pi + DASIN( sin )
            F = -x / ( 2.0D0*Pi - x )
         END IF

      END IF

      RETURN

      END
