C  File:  n1_s_primann.for

C  Autor:  G. Schnell

C  1.4.2008

C Berechnen Primann-Funktion pm(a1) mit sym. rek. Zahlen
C Ordnung: n=1,  Orientierung: o=s,  Grad: g=1
C Dimension d = d1 = 3
C ps2:  y =  1, e0 = -1, e1 = 0
C ps3:  y = -1, e0 =  1, e1 = 0
C pm( a0, a1 ) = ( exp[ps2](0,a0,a1) + exp[ps3](-a1,a0,0) ) / 2

      IMPLICIT NONE

      REAL*8 Y                ! Vorzeichen-Parameter
      REAL*8 Fe(2), e0, e1    ! Reduzierungs-Parameter

      EQUIVALENCE( Fe(1), e0 )
      EQUIVALENCE( Fe(2), e1 )            

      INTEGER*4 d1/ 3 /       ! Dimension d1 = 2g + 1
      INTEGER*4 J
      REAL*8 A0, A1
      REAL*8 Fa0( 3 )         ! a0: Anfang, Ende, Delta
      REAL*8 Fa1( 3 )         ! a1: Anfang, Ende, Delta

      REAL*8 Fa(-1:1), Fb(-1:1), Fc(-1:1), pm(-1:1)   ! a, b, c, pm

      CHARACTER*72 TXT

      INTEGER*4 IO_LI         ! I/O unit Liste
      INTEGER*4 IO_DAT        ! I/O unit Data
      CHARACTER*80 FILE_LI    ! Listfile
      CHARACTER*80 FILE_DAT   ! Datafile

      CHARACTER*(*) FMT1
      PARAMETER   ( FMT1 = '(1X,1P,5E14.6)' )

      CHARACTER*(*) FMTa
      PARAMETER   ( FMTa = '(1X,''a : '',1P,3E14.6)' )
      CHARACTER*(*) FMTb
      PARAMETER   ( FMTb = '(1X,''b : '',1P,3E14.6)' )
      CHARACTER*(*) FMTc
      PARAMETER   ( FMTc = '(1X,''c : '',1P,3E14.6)' )
      CHARACTER*(*) FMTpm
      PARAMETER   ( FMTpm= '(1X,''pm: '',1P,3E14.6)' )

      IO_DAT = 19   ! I/O Units setzen
      IO_LI  = 22   !

      ! Dateinamen:
      FILE_DAT = 'n1_s_primann.dat'
      FILE_LI  = 'n1_s_primann.ls'

      OPEN( UNIT=IO_DAT, FILE = FILE_DAT )
      OPEN( UNIT=IO_LI,  FILE = FILE_LI )

C  Daten von Datafile lesen

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! IO_DAT auf Datenzeile pos.
      READ(IO_DAT, '(A)' ) TXT          ! Text zur Aufgabe

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Werte zu a0
      READ(IO_DAT,*) ( Fa0(J),J=1,3 )   ! Anfang, Ende, Delta

      CALL UP_IN_POS( IO_DAT, IO_LI )   ! Werte zu a1
      READ(IO_DAT,*) ( Fa1(J),J=1,3 )   ! Anfang, Ende, Delta

      CLOSE ( IO_DAT )


      WRITE(IO_LI,*) 'Programm: n1_s_primann.exe'
      WRITE(IO_LI,*)
      WRITE(IO_LI,*) 'Primann-Funktion pm( a0, a1 )'
      WRITE(IO_LI,*)
      WRITE(IO_LI,2000) d1
 2000 FORMAT('Dimension d = ',I1)
      WRITE(IO_LI,2010)
      WRITE(IO_LI,2020)
 2010 FORMAT('Parametersatz ps2:  y =  1, e0 = -1, e1 = 0')
 2020 FORMAT('Parametersatz ps3:  y = -1, e0 =  1, e1 = 0')
      WRITE(IO_LI,*)
      WRITE(IO_LI,*)  
     + 'pm = ( exp[ps2](0,a0,a1) + exp[ps3](-a1,a0,0) ) / 2'
      WRITE(IO_LI,*)
      WRITE(IO_LI,*) TXT
      WRITE(IO_LI,*)
      WRITE(IO_LI,*) 'Werte a0: ( Anfang, Ende, Delta )'
      CALL DPUT( Fa0(1), IO_LI, FMT1, 1, 3, 0, 0 )
      WRITE(IO_LI,*)
      WRITE(IO_LI,*) 'Werte a1: ( Anfang, Ende, Delta )'
      CALL DPUT( Fa1(1), IO_LI, FMT1, 1, 3, 0, 0 )
      WRITE(IO_LI,*)

      WRITE(IO_LI,*) 'Ergebnis: pm( a0, a1 ) = ( b + c ) / 2'
      WRITE(IO_LI,*)

      e1      = 0.0D0

      DO A0 = Fa0(1), Fa0(2)+Fa0(3)/2, Fa0(3)

      DO A1 = Fa1(1), Fa1(2)+Fa1(3)/2, Fa1(3)

         Fa(-1) = 0.0D0
         Fa( 0) = A0
         Fa( 1) = A1
         CALL DPUT( Fa(0), IO_LI, FMTa, 1, 2, 0, 0 )

         y  =  1.0D0   ! ps2
         e0 = -1.0D0   !
         CALL sn1_exp( Fa, Fb, Y, Fe, IO_LI)
         CALL DPUT( Fb, IO_LI, FMTb, 1, d1, 0, 0 )

         Fa(-1) = -A1
         Fa( 0) = A0
         Fa( 1) = 0.0D0
         y  = -1.0D0   ! ps3
         e0 =  1.0D0   !
         CALL sn1_exp( Fa, Fc, Y, Fe, IO_LI)
         CALL DPUT( Fc, IO_LI, FMTc, 1, d1, 0, 0 )

         pm(-1) = ( Fb(-1) + Fc(-1) ) / 2.0D0
         pm( 0) = ( Fb( 0) + Fc( 0) ) / 2.0D0
         pm( 1) = ( Fb( 1) + Fc( 1) ) / 2.0D0
         CALL DPUT( pm, IO_LI, FMTpm, 1, d1, 0, 0 )
         WRITE(IO_LI,*)
      END DO   ! Schleife A1

      END DO   ! Schleife A0

      CLOSE ( IO_LI )

      STOP

      END
C --------------------------------------------------------
C  FILE_DAT = 'n1_s_primann.dat' fuers Lesen positionieren
C  und Kommentar- und Leer-Zeilen ueberlesen.

      SUBROUTINE UP_IN_POS( IO_IN, IO_LI )

      IMPLICIT NONE

      INTEGER*4 IO_IN, IO_LI, I
      CHARACTER*136 REC

   10 READ( IO_IN, '(A)', ERR=800, END=900 ) REC
      IF( REC(1:1) .EQ. '!' ) GO TO 10   ! Kommentarzeile ueberlesen
      DO 20, I = 1, 136
         IF( REC(I:I) .NE. ' ' ) THEN
            BACKSPACE IO_IN
            RETURN                       ! Datenzeile gefunden
         END IF
   20 CONTINUE
      GO TO 10                           ! Leerzeile ueberlesen

  800 WRITE( IO_LI, * ) 'Error beim Lesen von n1_s_primann.dat'
      STOP

  900 WRITE( IO_LI, * ) 'EOF erreicht beim Lesen von n1_s_primann.dat'
      STOP
      END
C  ---------- DPUT ---------------------------
C  Drucken Matrix oder Vektor FD[M,N],  Real*8
C  FD    = Matrix[M,N]
C  IOU   = Output-Unit
C  IVOR  = Anzahl Leer-Zeilen vor dem Drucken
C  INACH = Anzahl Leer-Zeilen nach dem Drucken
C  HFORM = Vektor der das Druck-Format enthaelt

      SUBROUTINE DPUT( FD, IOU, HFORM, M, N, IVOR, INACH )

      IMPLICIT NONE

      INTEGER*4 IOU
      INTEGER*4 N, M, IVOR, INACH, I, J
      REAL*8 FD(M,N)
      CHARACTER*(*) HFORM

      IF( IVOR .LE. 0 ) GO TO 9

      DO 5, I = 1, IVOR
         WRITE(IOU,2000)
    5 CONTINUE

    9 CONTINUE

      DO 10, I = 1, M
       WRITE(IOU,HFORM) ( FD(I,J), J = 1, N )
   10 CONTINUE

      IF( INACH .LE. 0 ) RETURN

      DO 15, I = 1, INACH
         WRITE(IOU,2000)
   15 CONTINUE

      RETURN

 2000 FORMAT(2H  )
      END
C  ----------------------------------------------------
C  Berechnen Exponentialfunktion einer sym. rek. Zahl a

C  exp( a ) = 1 + a/1! + a^2/2! + a^3/3! + ...
C  d1 = 3

      SUBROUTINE sn1_exp( Fa, Fc, Y, Fe, IO_LI )

      IMPLICIT NONE

      INTEGER*4 IO_LI
      REAL*8 Fa(3), Fc(3), Fe(2), Y

      INTEGER*4 d1/ 3 /
      REAL*8 MA( 3, 3 )
      REAL*8 Fb( 3 ), DI
      REAL*8 Fz( 3 ), Fb_alt( 3 )

      INTEGER*4 I, J
      LOGICAL*4 LOKAY

      ! n1-Matrix MA erstellen
      CALL GEN_MA_Y( Fa, MA, Y, Fe )

      ! Fc = Fa, Fz = Fa,  Fb_alt = 0
      DO I = 1, d1
         Fc(I) = Fa(I)
         Fz(I) = Fa(I)
         Fb_alt(I) = 0.0D0
      END DO
         Fc(2) = Fc(2) + 1.0D0   ! e^a = 1 + a/1! + ...

      DO I = 2, 100   ! 100 Reihenglieder sollten reichen
         DI = I
         CALL MLT_M_V( d1, MA, Fz, Fb )   ! Fb = MA * Fz
         LOKAY = .TRUE.
         DO J = 1, d1
            Fz(J) = Fb(J) / DI
            Fc(J) = Fc(J) + Fz(J)
            IF( Fc(J) .NE. Fb_alt(J) ) LOKAY = .FALSE.
            Fb_alt(J) = Fc(J)
         END DO
         IF( LOKAY ) GO TO 100   ! Ergebnis steht
      END DO
         ! Ergebnis nicht korrekt !
         WRITE(IO_LI,*) 'Ergebnis exp nicht korrekt nach 100 Schritten.'

  100 CONTINUE

      RETURN

      END
C  ========================
C  Matrix Ma zu a erstellen
C  Die Dimension d1 von a ist 3
C  Y ist Vorzeichen-Parameter
C  E0, E1 sind Reduktions-Parameter
C  A0, A1, A2 stehen in Fa

      SUBROUTINE GEN_MA_Y( Fa, ma, Y, Fe )

      IMPLICIT NONE

      REAL*8 Fa(3), Ma(3,3), Y, Fe(2)

      REAL*8 E0, E1

      E0 = Fe(1)
      E1 = Fe(2)

      MA(1,1) =  ( Fa(2) - Fa(1) * E1 / E0 ) / Y
      MA(1,2) =    Fa(1) / Y
      MA(1,3) =    0.0D0
      MA(2,1) =    Fa(3) - Fa(1) / E0
      Ma(2,2) =    Fa(2)
      Ma(2,3) =    Fa(1) - Fa(3) * E0
      Ma(3,1) =    0.0D0
      Ma(3,2) =    Fa(3) * Y
      Ma(3,3) = ( Fa(2) - Fa(3) * E1 ) * Y

      RETURN

      END
C  ----------------------------------------------------
C  Multiplizieren Matrix MA mit Vektor B, Ergebnis in C
C  d1 ist Dimension von Ma, B, C.

      SUBROUTINE MLT_M_V( d1, MA, B, C )

      IMPLICIT NONE

      INTEGER*4 d1,   I, J
      REAL*8    MA(d1,d1), B(d1), C(d1)

      DO I = 1, d1
         C(I) = 0.0D0
         DO J = 1, d1
            C(I) = C(I) + MA(I,J) * B(J)
         END DO
      END DO

      RETURN
      END
